#######################################################################
# This script compares the speed of the computation of a polynomial
# for different (numpy and numexpr) in-memory paradigms.
#
# Author: Francesc Alted
# Date: 2011-09-11
#######################################################################

from time import time
import numpy as np
import numexpr as ne

expr = ".25*x**3 + .75*x**2 - 1.5*x - 2"  # 1) the polynomial to compute
#expr = "((.25*x + .75)*x - 1.5)*x - 2"   # 2) a computer-friendly polynomial
#expr = "x"                                # 3) the identity function
#expr = "sin(x)**2+cos(x)**2"             # 4) a transcendental function
N = 10*1000*1000            # the number of points to compute expression
x = np.linspace(-1, 1, N)   # the x in range [-1, 1]

what = "numpy"              # uses numpy for computations
#what = "numexpr"           # uses numexpr for computations


def compute(nt):
    """Compute the polynomial with `nt` threads."""
    global expr
    ne.set_num_threads(nt)       # the number of threads for numexpr computations
    if what == "numpy":
        if "sin" in expr:
            # Trick to allow numpy evaluate this
            expr = "np.sin(x)**2+np.cos(x)**2"
        elif expr == "x":
            # Trick to force a copy with NumPy
            y = x.copy()
        y = eval(expr)
    else:
        y = ne.evaluate(expr)
    return len(y)


if __name__ == '__main__':
    print "Computing: '%s' using %s with %d points" % (expr, what, N)
    for nt in range(1):   # set the number of threads you want to test
        t0 = time()
        result = compute(nt+1)
        ts = round(time() - t0, 3)
        print "*** Time elapsed:", ts, "s"
